package com.hello2morrow.foundation.utilities

import de.schlichtherle.truezip.file.TFile

public class SectionedData
{
    private final Map<String, String> m_data;

    private SectionedData(Map<String, String> data)
    {
        m_data = data;
    }

    public String getRawContent(String section)
    {
        return m_data.get(section)
    }

    public void setRawContent(String section, String content)
    {
        m_data.put(section, content)
    }

    public boolean hasSection(String section)
    {
        return m_data.get(section) != null
    }


    public boolean removeSection(String section)
    {
        return m_data.remove(section) != null
    }

    public Collection<String> getSections()
    {
        return m_data.keySet()
    }

    public int size()
    {
        return m_data.size()
    }

    public void writeToFile(TFile file)
    {
        file.setText(toString())
    }

    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();

        for (String k : m_data.keySet())
        {
            sb.append('[').append(k).append(']\n');
            sb.append(m_data.get(k));
            sb.append('\n\n');
        }
        return sb.toString();
    }

    public static SectionedData fromFile(TFile file)
    {
        if (!file.canRead())
        {
            return null;
        }
        return new SectionedData(parseContent(file.getText()))
    }

    public static SectionedData fromStream(InputStream input)
    {
        return new SectionedData(parseContent(input.getText()))
    }

    public static SectionedData fromFile(String fileName)
    {
        return fromFile(new TFile(fileName))
    }

    public static SectionedData fromString(String data)
    {
        return new SectionedData(parseContent(data))
    }

    private static Map<String, String> parseContent(String text)
    {
        Map<String, String> result = new LinkedHashMap<>()
        String section = null;
        String content = "";

        text.eachLine
        {
            def trimmedLine = it.trim()

            if (trimmedLine ==~ /\[\w+( \w+)*\]/)
            {
                if (section != null)
                {
                    result.put(section, content);
                }
                section = trimmedLine.substring(1, trimmedLine.length()-1)
                content = ""
            }
            else if (!it.startsWith("#"))
            {
                content += it+'\n'
            }
        }
        if (section != null)
        {
            result.put(section, content)
        }
        return result
    }

    public static SectionedData fromMap(Map<String, String> dataMap)
    {
        return new SectionedData(dataMap);
    }
}
